#include "libmpib.h"
#include "mpib_coll_benchmarks.h"
#include "mpib_output.h"


int MPIB_measure_max(MPIB_coll_container* container, MPI_Comm comm, int root,
		int M, MPIB_precision precision, MPIB_result* result) {
	double* T = (double*)malloc(sizeof(double) * precision.max_reps);
	double wtick;
	MPIB_max_wtick(comm, &wtick);
	container->initialize(container, comm, root, M);

	int stop = 0;
	double sum = 0;
	int reps = 0;
	double ci = 0;
	while (!stop && reps < precision.max_reps) {
		MPI_Barrier(comm); MPI_Barrier(comm);
		double time = MPI_Wtime();
		int res = container->execute(container, comm, root, M);
		time = MPI_Wtime() - time;
		if (res != 0) return res;
		MPI_Allreduce(&time, &T[reps], 1, MPI_DOUBLE, MPI_MAX, comm);
		sum += T[reps];
		reps++;
		if (reps >= precision.min_reps && reps > 2)
			stop = (ci = MPIB_ci(precision.cl, reps, T)) * reps / sum < precision.eps;
	}
	container->finalize(container, comm, root);
	*result = (MPIB_result){M, sum / reps, wtick, reps, ci};
	free(T);
	return 0;
}

int MPIB_measure_max_adaptive(MPIB_coll_container* container, MPI_Comm comm, int root,
		MPIB_msgset msgset,	MPIB_precision precision, int* count, MPIB_result** results) {
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == root) {
		if (MPIB_verbose)
			fprintf(stderr, "#adaptive max benchmark root = %d:", root);
		*count = 0;
		*results = NULL;
	}
	int stride = msgset.min_stride;
	int c = 0;
	int M = msgset.min_size;
	// use the same array and wtick value for communication experiments with different message sizes
	double* T = (double*)malloc(sizeof(double) * precision.max_reps);
	double wtick;
	MPIB_max_wtick(comm, &wtick);
	while (M < msgset.max_size && c < msgset.max_num) {
		if ((rank == root) && MPIB_verbose)
			fprintf(stderr, " %d", M);
		container->initialize(container, comm, root, M);
		int stop = 0;
		double sum = 0;
		int reps = 0;
		double ci = 0;
		while (!stop && reps < precision.max_reps) {
			MPI_Barrier(comm); MPI_Barrier(comm);
			double time = MPI_Wtime();
			int res = container->execute(container, comm, root, M);
			time = MPI_Wtime() - time;
			if (res != 0) return res;
			MPI_Allreduce(&time, &T[reps], 1, MPI_DOUBLE, MPI_MAX, comm);
			sum += T[reps];
			reps++;
			if (reps >= precision.min_reps && reps > 2)
				stop = (ci = MPIB_ci(precision.cl, reps, T)) * reps / sum < precision.eps;
		}
		container->finalize(container, comm, root);
		if (rank == root) {
			*count = c + 1;
			*results = (MPIB_result*)realloc(*results, sizeof(MPIB_result) * (c + 1));
			*results[c] = (MPIB_result){M, sum / reps, wtick, reps, ci};
		}
		if (MPIB_diff(*results[c], &*results[c - 2]) < msgset.max_diff)
			stride *= 2;
		else if (stride > msgset.min_stride)
			stride /= 2;
		c++;
	}
	free(T);
	if ((rank == root) && MPIB_verbose)
		fprintf(stderr, "\n");
	return 0;
}
