#include "libmpib.h"
#include "mpib_measurement.h"
#include <math.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_statistics_double.h>

void MPIB_Comm(MPI_Comm comm, MPI_Comm* newcomm) {
	int count = 0;
	int size, rank;
	MPI_Comm_size(comm, &size);
	MPI_Comm_rank(comm, &rank);
	int actual_len;
        char name[MPI_MAX_PROCESSOR_NAME];
	char recv_buffer[size][MPI_MAX_PROCESSOR_NAME];
        MPI_Get_processor_name(name, &actual_len);
        MPI_Allgather(name, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, recv_buffer, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, comm);
	qsort(recv_buffer, size, MPI_MAX_PROCESSOR_NAME, (int(*)(const void*, const void*))strcmp);
	int i;
	int color = MPI_UNDEFINED;
	int key = MPI_UNDEFINED;
	int partition = 0;
	/*
	Make every process find it's position in the sorted list of hostnames and set
	the key accordingly. Only set the color if we do not have a repetitions of processes 
	running on the same host. In case many processes per host, only the first process on
	this host will not die, i.e. will be in a communicator
	*/
	for (i = 0 ; i < size; i++) {
		if (i > 0) {
			if (strcmp(recv_buffer[i-1], recv_buffer[i])) 
				partition++;
		}
		if (!strcmp(recv_buffer[i], name)) {
			key = i;
			color = partition;
			break;
		}
	}
	MPI_Comm new_comm;
	//create a communicator per hostname and choose only lowest rank process per hostname to represent it
	MPI_Comm_split(comm, color, rank, &new_comm);
	int new_rank;
	MPI_Comm_rank(new_comm, &new_rank);
	int new_size;
	MPI_Comm_size(new_comm, &new_size);
	if (new_rank != 0)
		color = MPI_UNDEFINED;
	else
		color = 0;
	MPI_Comm_free(&new_comm);
	//the final communicator includes one process per hostname and is sorted increasingly according to hostnames
	MPI_Comm_split(comm, color, key, newcomm);
}

double MPIB_diff(MPIB_result result, MPIB_result results[2]) {
	return fabs(1 - result.T * (results[1].M - results[0].M) /
		(results[1].T * (result.M - results[0].M) - results[0].T * (result.M - results[1].M)));
}

void MPIB_max_wtick(MPI_Comm comm, double* wtick) {
	double tmp = MPI_Wtick();
	MPI_Allreduce(&tmp, wtick, 1, MPI_DOUBLE, MPI_MAX, comm); 
}

double MPIB_ci(double cl, int reps, double* T) {
	return fabs(gsl_cdf_tdist_Pinv(cl, reps - 1)) * gsl_stats_sd(T, 1, reps) / sqrt(reps); 
}
