#ifndef MPIB_COLL_H_
#define MPIB_COLL_H_

/*!
 * \page design
 * \section mpib_coll Collectives library
 * A shared library \c libmpib_coll.so implements different algorithms of collectives
 * - \ref basic_collectives (mostly linear)
 * - \ref tree_collectives (depends on the Boost C++ libraries)
 *
 * The command-line arguments:
 * - \b verbose verbose mode (default: no)
 * - \b sgv scatterv/gatherv mode
 *   - \e 0 propagated on duplicated communicator (default)
 *   - \e 1 broadcasted counts
 *   - \e 2 propagated with tags: not safe
 *   - \e 3 everywhere-defined counts: not standard
 *
 * \subsection tools Tools
 * - \ref tools/collective_test.c - verifies implementations of collective communication operations
 *
 * An example showing how collective operations implemented in the library can be used in applications
 * instead of their native counterparts \ref tests/substitute.c.
 */

#include "mpib_coll_config.h"

/*!
 * TODO docs
 */
typedef enum {ASC, DESC} MPIB_sort_order;

/*!
 * TODO docs
 */
typedef enum {L2R, R2L} MPIB_child_traverse_order;

#include "mpib_basic_collectives.h"
#ifdef HAVE_LIBBOOST_GRAPH
#include "mpib_tree_collectives.h"
#endif /*HAVE_LIBBOOST_GRAPH*/

#endif /* MPIB_COLL_H_ */
