#include "libmpib_coll.h"

int MPIB_Scatter_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype, 
	void* recvbuf, int recvcount, MPI_Datatype recvtype, 
	int root, MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == root) {
		int size;
		MPI_Comm_size(comm, &size);
		MPI_Aint extent;
		MPI_Type_extent(sendtype, &extent);
		int inc = sendcount * extent;
		int i;
		char* ptr;
		for (i = 0, ptr = (char*)sendbuf; i < size; i++, ptr += inc) {
			if (i == root) {
				MPI_Aint extent;
				MPI_Type_extent(recvtype, &extent);
				memcpy(recvbuf, ptr, recvcount * extent);
			} else
				MPI_Send(ptr, sendcount, sendtype, i, 0, comm);
		}
	} else
		MPI_Recv(recvbuf, recvcount, recvtype, root, 0, comm, MPI_STATUS_IGNORE);
	return MPI_SUCCESS;
}

int MPIB_Gather_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype, 
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	if (rank == root) {
		int size;
		MPI_Comm_size(comm, &size);
		MPI_Aint extent;
		MPI_Type_extent(recvtype, &extent);
		int inc = recvcount * extent;
		int i;
		char* ptr;
		for (i = 0, ptr = (char*)recvbuf; i < size; i++, ptr += inc) {
			if (i == root)
				memcpy(ptr, sendbuf, inc);
			else
				MPI_Recv(ptr, recvcount, recvtype, i, 0, comm, MPI_STATUS_IGNORE);
		}
	} else
		MPI_Send(sendbuf, sendcount, sendtype, root, 0, comm);
	return MPI_SUCCESS;
}
